<?php

/*
Plugin Name: Section widget
Plugin URI: http://blogs.ubc.ca/support/plugins/section-widget/
Description: Display arbitrary text or HTML on certain sections of your site.
Author: Godfrey Chan (UBC OLT)
Version: 2.03
Author URI: http://www.chancancode.com/
*/

/**
 * Section widget class
 */

class WP_Widget_Section extends WP_Widget {
    
	function WP_Widget_Section() {
		$widget_ops = array('classname' => 'widget_section', 'description' => __('Display arbitrary text or HTML on certain sections of your site.'));
		$control_ops = array('width' => 400);
		$this->WP_Widget('section', __('Section'), $widget_ops, $control_ops);
	}
    
	function widget( $args, $instance ) {
		extract($args);
				
		// Most of the conditional tags will always return true when passed an empty array. So we need to pad them will some junk.
		$instance['special-pages'][] = -1;
		$instance['pages'][] = -1;
		$instance['categories'][] = -1;
		$instance['tags'][] = -1;
		
		$should_display = false;
		
		foreach($instance['special-pages'] as $key) {
		    switch($key){
		        case "front":
		            if(is_front_page())
		                $should_display = true;
		            break;
                case "category":
                    if(is_category($instance['categories']))
                        $should_display = true;
		            break;
		        case "tag":
		            if(is_tag($instance['tags']))
		                $should_display = true;
		            break;
		        case "date":
		            if(is_date())
		                $should_display = true;
		            break;
		        case "page":
		            // Exclude the front page to avoid confusion
		            if(!is_front_page() && is_page())
		                $should_display = true;
		            break;
		        case "post":
		            if(is_single())
		                $should_display = true;
		            break;
		        case "comment":
		            // Exclude the front page to avoid confusion
		            if(!is_front_page() && is_singular() && comments_open())
		                $should_display = true;
		            break;
		        case "author":
		            if(is_author())
		                $should_display = true;
		            break;
		    }
		    
		    if($should_display) break;
		}
		
		if(!$should_display) {
		    if(is_page($instance['pages']))
		        $should_display = true;	
		    elseif(is_singular() && in_category($instance['categories']))
		        $should_display = true;
		    elseif(is_singular() && has_tag($instance['tags']))
		        $should_display = true;
		}
        
        if($should_display) {
    		echo $before_widget;
    		?>
    		    <?php
    		        if($instance['display-title'])
    		            echo $before_title, apply_filters('widget_title', $instance['title']), $after_title;
    		    ?>
    			<div class="sectionwidget"><?php echo apply_filters('widget_text', $instance['body']); ?></div>
    		<?php
    		echo $after_widget;
    	}
	}

	function update( $new_instance, $old_instance ) {
	    // Mostly borrowed from text widget
		$instance = $old_instance;
		$instance['title'] = strip_tags($new_instance['title']);
		$instance['display-title'] = (bool) $new_instance['display-title'];
		$instance['special-pages'] = is_array($new_instance['special-pages'])? $new_instance['special-pages'] : array();
		$instance['pages'] = is_array($new_instance['pages'])? $new_instance['pages'] : array();
		$instance['categories'] = is_array($new_instance['categories'])? $new_instance['categories'] : array();
		$instance['tags'] = is_array($new_instance['tags'])? $new_instance['tags'] : array();
		if ( current_user_can('unfiltered_html') )
			$instance['body'] =  $new_instance['body'];
		else
			$instance['body'] = wp_filter_post_kses( $new_instance['body'] );
		return $instance;
	}

	function form( $instance ) {
	    // Provide the defaults here
		$instance = wp_parse_args( (array) $instance, array( 'title' => '', 'display-title' => true, 'body' => '', 'special-pages' => array(), 'pages' => array(), 'categories' => array(), 'tags' => array()) );
		
		$title = strip_tags($instance['title']);
		$display_title = (bool) $instance['display-title'];
		$special_pages = $instance['special-pages'];
		$pages = $instance['pages'];
		$categories = $instance['categories'];
		$tags = $instance['tags'];
		$body = format_to_edit($instance['body']);
?>
		<p>
		    <label for="<?php echo $this->get_field_id('title'); ?>"><?php _e('Title:'); ?></label>
		    <input class="widefat" id="<?php echo $this->get_field_id('title'); ?>" name="<?php echo $this->get_field_name('title'); ?>" type="text" value="<?php echo esc_attr($title); ?>" />
		    <input id="<?php echo $this->get_field_id('display-title'); ?>" name="<?php echo $this->get_field_name('display-title'); ?>" type="checkbox" <?php if($display_title) echo 'checked="checked"'; ?>" />
		    <label for="<?php echo $this->get_field_id('display-title'); ?>"><?php _e('Display title'); ?></label>
		</p>
        <p>
            <label><?php _e('Display this widget on:'); ?></label>
        </p>
        <div id="<?php echo $this->get_field_id('tabs-container'); ?>" class="section-widget-tabs-container">
            <ul class="section-widget-tabs">
                <li class="section-widget-tab"><a href="#<?php echo $this->get_field_id('special-pages'); ?>">Special Pages</a></li>
                <li class="section-widget-tab"><a href="#<?php echo $this->get_field_id('pages-tab'); ?>">Pages</a></li>
                <li class="section-widget-tab"><a href="#<?php echo $this->get_field_id('categories-tab'); ?>">Categories</a></li>
                <li class="section-widget-tab"><a href="#<?php echo $this->get_field_id('tags-tab'); ?>">Tags</a></li>
            </ul>
            <div id="<?php echo $this->get_field_id('special-pages'); ?>" class="section-widget-tab-panel">
                <ul id="<?php echo $this->get_field_id('special-pages'); ?>">
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-front'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="front" <?php if(in_array('front', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-front'); ?>">Front Page</label>
                            <p>
                                Display this widget on the front page, as determined by your site's <a href="<?php echo admin_url('options-reading.php'); ?>">Reading Settings</a>.
                            </p>
                        </div>
                    </li>
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-category'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="category" <?php if(in_array('category', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-category'); ?>">Category Archive Pages</label>
                            <p>
                                Display this widget on a category archive page if the corresponding category is selected in the <strong>Categories</strong> tab.
                            </p>
                        </div>
                    </li>
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-tag'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="tag" <?php if(in_array('tag', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-tag'); ?>">Tag Archive Pages</label>
                            <p>
                                Display this widget on a tag archive page if the corresponding tag is selected in the <strong>Tags</strong> tab.
                            </p>
                        </div>
                    </li>
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-date'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="date" <?php if(in_array('date', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-tag'); ?>">Date-Based Archive Pages</label>
                            <p>
                                Display this widget on all date-based archive pages (i.e. all monthly, yearly, daily or time-based archives).
                            </p>
                        </div>
                    </li>
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-page'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="page" <?php if(in_array('page', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-page'); ?>">All Pages</label>
                            <p>
                                Display this widget on all pages.
                            </p>
                        </div>
                    </li>
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-post'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="post" <?php if(in_array('post', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-post'); ?>">All Posts</label>
                            <p>
                                Display this widget on all posts.
                            </p>
                        </div>
                    </li>
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-comment'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="comment" <?php if(in_array('comment', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-comment'); ?>">Commentable Pages/Posts</label>
                            <p>
                                Display this widget on all pages/posts where comments are allowed.
                            </p>
                        </div>
                    </li>
                    <li>
                        <div class="section-widget-checklist-entry">
                            <input type="checkbox" class="level-1" id="<?php echo $this->get_field_id('special-pages-author'); ?>" name="<?php echo $this->get_field_name('special-pages'); ?>[]" value="author" <?php if(in_array('author', $special_pages)) echo 'checked="checked" ' ?>/>
                            <label for="<?php echo $this->get_field_id('special-pages-author'); ?>">Author Pages</label>
                            <p>
                                Display this widget on all author pages.
                            </p>
                        </div>
                    </li>
                </ul>
            </div>
            <div id="<?php echo $this->get_field_id('pages-tab'); ?>" class="section-widget-tab-panel">
                <?php my_checklist('pages', array('id' => $this->get_field_id('pages'), 'name' => $this->get_field_name('pages'), 'selected' => $pages)); ?>
            </div>
            <div id="<?php echo $this->get_field_id('categories-tab'); ?>" class="section-widget-tab-panel">
                <?php my_checklist('categories', array('id' => $this->get_field_id('categories'), 'name' => $this->get_field_name('categories'), 'selected' => $categories)); ?>
            </div>
            <div id="<?php echo $this->get_field_id('tags-tab'); ?>" class="section-widget-tab-panel">
                <?php my_checklist('tags', array('id' => $this->get_field_id('tags'), 'name' => $this->get_field_name('tags'), 'selected' => $tags)); ?>
            </div>
        </div>
        <p>
		    <textarea class="widefat" rows="16" cols="20" id="<?php echo $this->get_field_id('body'); ?>" name="<?php echo $this->get_field_name('body'); ?>"><?php echo $body; ?></textarea>
		</p>
		<?php /* The new AJAX submit in WP2.8 actually reload the content of our div, so you'd have to re-init it everytime. */ ?>
		<script type="text/javascript">
		    if(typeof sectionWidgetLoaded != 'undefined')
		        sectionWidgetCreateTabs(jQuery('#<?php echo $this->get_field_id('tabs-container'); ?>'));
		</script>
<?php
	}
}

/*
    Unfortunately, things are not as easy as it should have been. I borrowed some code from wordpress to display our checklists.
    Check out the sourse for the Walker class (in /wp-includes/classes.php -> line 714 as of 2.8) and the wp_category_checklist()
    function (in /wp-admin/includes/template.php -> line 490 as of 2.8) to make sense of the following code.
*/

function my_checklist($mode = 'pages', $args = '') {

    switch($mode) {
        case 'categories':
            $defaults = array( 'selected' => array(), 'echo' => 1, 'id' => 'page_id', 'name' => 'page_id' );
            $data = get_categories('get=all');
            $walker = new MyCategoryChecklistWalker;
            break;
        
        case 'tags':
            $defaults = array( 'selected' => array(), 'echo' => 1, 'id' => 'tag_id', 'name' => 'tag_id' );
            $data = get_tags('get=all');
            $walker = new MyTagChecklistWalker;
            break;
        
        case 'pages':
        default:
            $defaults = array( 'selected' => array(), 'echo' => 1, 'id' => 'category_id', 'name' => 'category_id' );
            $data = get_pages('get=all');
            $walker = new MyPageChecklistWalker;
            break;
    }
    
    $r = wp_parse_args( $args, $defaults );
	extract( $r, EXTR_SKIP );
        
	if ( ! empty($data) ) {
		$output = "<ul id=\"$id\">\n";
	    $output .= $walker->walk($data, 0, $r);
		$output .= "</ul>\n";
	} else {
	    $output = "No $mode found.";
	}

	$output = apply_filters('wp_dropdown_pages', $output);

	if ( $echo ) echo $output;
        
	return $output;
}

class MyChecklistWalker extends Walker {
	
	function start_lvl(&$output, $depth, $args) {
    	$indent = str_repeat("\t", $depth);
    	$output .= "$indent<ul class='children'>\n";
    }
    
    function end_lvl(&$output, $depth, $args) {
    	$indent = str_repeat("\t", $depth);
    	$output .= "$indent</ul>\n";
    }
	
	function end_el(&$output, $page, $depth, $args) {
    	$output .= "</li>\n";
    }
}

class MyPageChecklistWalker extends MyChecklistWalker {

	var $tree_type = 'page';
	var $db_fields = array ('parent' => 'post_parent', 'id' => 'ID');
	
	function start_el(&$output, $page, $depth, $args) {
	    extract($args);
	    
		$pad = str_repeat('&nbsp;', $depth * 4);
        $id = "{$id}-{$page->ID}";
        $name .= '[]';
        $title = esc_html($page->post_title);
                        
		$output .= "<li><div class=\"section-widget-checklist-entry\">$pad<input type=\"checkbox\" class=\"level-$depth\" id=\"$id\" name=\"$name\" value=\"$page->ID\"";
		if ( in_array($page->ID, $selected) )
			$output .= ' checked="checked"';
		$output .= " /> <label for=\"$id\">$title</label></div>\n";
	}
}

class MyCategoryChecklistWalker extends MyChecklistWalker {

	var $tree_type = 'category';
	var $db_fields = array ('parent' => 'parent', 'id' => 'term_id');
	
	function start_el(&$output, $category, $depth, $args) {
	    extract($args);
	    
		$pad = str_repeat('&nbsp;', $depth * 4);
        $id = "{$id}-{$category->term_id}";
        $name .= '[]';
        $title = esc_html($category->name);
        
		$output .= "<li><div class=\"section-widget-checklist-entry\">$pad<input type=\"checkbox\" class=\"level-$depth\" id=\"$id\" name=\"$name\" value=\"$category->term_id\"";
		if ( in_array($category->term_id, $selected) )
			$output .= ' checked="checked"';
		$output .= " /> <label for=\"$id\">$title</label></div>\n";
	}
}

class MyTagChecklistWalker extends Walker {

	var $tree_type = 'tag';
	var $db_fields = array ('parent' => 'parent', 'id' => 'term_id');
	
	function start_el(&$output, $tag, $depth, $args) {
	    extract($args);
	    
        $id = "{$id}-{$tag->term_id}";
        $name .= '[]';
        $title = esc_html($tag->name);
                
		$output .= "<li><div class=\"section-widget-checklist-entry\"><input type=\"checkbox\" class=\"level-$depth\" id=\"$id\" name=\"$name\" value=\"$tag->term_id\"";
		if ( in_array($tag->term_id, $selected) )
			$output .= ' checked="checked"';
		$output .= " /> <label for=\"$id\">$title</label></div>\n";
	}
}

/* Finally, register this plugin/widget with wordpress. */

function widget_section_init() {
    if(is_admin()) {
        wp_enqueue_script('jquery-ui-tabs');
        wp_enqueue_script('section-widget', plugins_url('section-widget/section-widget.js'));
        wp_enqueue_style('section-widget', plugins_url('section-widget/section-widget.css'));
    }
    
	register_widget('WP_Widget_Section');
}

### Function: Init Section Widget
add_action('widgets_init', 'widget_section_init');

?>
